@@ -34,6 +34,9 @@ our %NAMESPACES = (
    x_delay     => 'jabber:x:delay',
    delay       => 'urn:xmpp:delay',
    ping        => 'urn:xmpp:ping',
+
+   vcard       => 'vcard-temp',
+   vcard_upd   => 'vcard-temp:x:update',
 );
 
 =head1 NAME
@@ -84,6 +87,8 @@ which make handling of namepsaces a bit easier:
 
    ping        => urn:xmpp:ping
 
+   vcard       => vcard-temp
+
 =head1 FUNCTIONS
 
 =over 4
